"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const TJS = require("typescript-json-schema");
const globLib = require("glob");
const args = require("commander");
const ts = require("typescript");
const fs = require("fs-promise");
const path = require("path");
const mkdirp = require("mkdirp");
const chokidar = require("chokidar");
let watcher = null;
args.version("0.1.7")
    .usage("[options]")
    .option("--tsconfig <path>", "Path to tsconfig.json.")
    .option("--srcRoot <path>", "Path to root directory with TypeScript files.")
    .option("--srcFilePattern <glob>", "E.g. **/*.schema.ts - specifies which TypeScript files to process. This is relative to --srcRoot.")
    .option("--targetRoot <path>", "Path to target directory where *.json schema files will be generated.")
    .option("--defaultProps", "Create default properties definitions.", false)
    .option("--required", "Create required array for non-optional properties.", false)
    .option("--strictNullChecks", "Make values non-nullable by default.", false)
    .option("--watch", "Re-generate Json schema file on change to TypeScript file.", false)
    .option("--verbose", "Show info for each generated file.", false)
    .parse(process.argv);
validateArgumentsGiven(["tsconfig", "srcRoot", "srcFilePattern", "targetRoot"]);
run();
function validateArgumentsGiven(argumentNames) {
    argumentNames.forEach(argumentName => {
        if (!args[argumentName]) {
            console.error(`--${argumentName} is required.`);
        }
    });
    if (argumentNames.some(argumentName => !args[argumentName])) {
        console.info("Use --help to display usage.");
        process.exit(2);
    }
}
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        const srcFilesGlob = path.join(args["srcRoot"], args["srcFilePattern"]);
        const files = yield glob(srcFilesGlob);
        if (args["watch"]) {
            setUpWatching(srcFilesGlob, files);
        }
        else {
            const success = yield generateManyFiles(files);
            process.exit(success ? 0 : 1);
        }
    });
}
function setUpWatching(srcFilesGlob, allMatchingFiles) {
    let modifiedFiles = new Set(allMatchingFiles);
    console.info("# Setting up watching on " + srcFilesGlob);
    watcher = chokidar.watch(srcFilesGlob, { persistent: true, awaitWriteFinish: true });
    watcher.on("add", queueChange);
    watcher.on("change", queueChange);
    function queueChange(path) {
        console.info("# Event for " + path);
        modifiedFiles.add(path.replace(new RegExp("\\\\", "g"), "/"));
        processChanges();
    }
    const debounce = require("debounce");
    const processChanges = debounce(() => {
        console.info("# Generating many files " + modifiedFiles.size);
        generateManyFiles(Array.from(modifiedFiles));
        modifiedFiles.clear();
    }, 200);
    processChanges();
}
function generateManyFiles(sourceFilePaths) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!sourceFilePaths.length) {
            return Promise.resolve(true);
        }
        const program = programFromConfig(sourceFilePaths, args["tsconfig"]);
        const generatorSettings = {
            useDefaultProperties: args["defaultProps"],
            generateRequired: args["required"],
            strictNullChecks: args["strictNullChecks"]
        };
        const generator = TJS.buildGenerator(program, generatorSettings);
        if (!generator) {
            console.error('Json schema generation has failed due to TypeScript compilation errors.');
            return Promise.resolve(false);
        }
        let numberOfSuccessful = 0;
        for (let i = 0; i < sourceFilePaths.length; i++) {
            try {
                yield generateOneFile(sourceFilePaths[i], generator);
                numberOfSuccessful++;
                if (args["verbose"]) {
                    console.log(`Generated Json schema for ${sourceFilePaths[i]}.`);
                }
            }
            catch (error) {
                console.error(`Failed to generate Json schema for ${sourceFilePaths[i]}: ${error}.`);
            }
        }
        ;
        if (!args["verbose"]) {
            console.log(`Generated Json schema for ${numberOfSuccessful} file${numberOfSuccessful > 1 ? "s" : ""} matching ${args["srcFilePattern"]}.`);
        }
        return numberOfSuccessful == sourceFilePaths.length;
    });
}
function generateOneFile(sourceFilePath, generator) {
    return __awaiter(this, void 0, void 0, function* () {
        const sourceFileName = path.basename(sourceFilePath);
        const coreName = sourceFileName.split(".", 1)[0];
        const interfaceName = coreName.charAt(0).toUpperCase() + coreName.slice(1);
        const outputFileName = sourceFileName.slice(0, sourceFileName.lastIndexOf(".")) + ".json";
        const outputDirName = path.join(args["targetRoot"], path.relative(args["srcRoot"], path.dirname(sourceFilePath)));
        const outputFilePath = path.join(outputDirName, outputFileName);
        const schemaDefinition = generator.getSchemaForSymbol(interfaceName);
        mkdirp.sync(outputDirName);
        yield fs.writeJson(outputFilePath, schemaDefinition);
    });
}
function programFromConfig(fileNames, configFileName) {
    const result = ts.parseConfigFileTextToJson(configFileName, ts.sys.readFile(configFileName));
    const configObject = result.config;
    const configParseResult = ts.parseJsonConfigFileContent(configObject, ts.sys, path.dirname(configFileName), {}, configFileName);
    const options = configParseResult.options;
    options.noEmit = true;
    options.skipLibCheck = true;
    delete options.out;
    delete options.outDir;
    delete options.outFile;
    delete options.declaration;
    return ts.createProgram(fileNames, options);
}
function glob(pattern) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            globLib(pattern, (error, matches) => {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(matches);
                }
            });
        });
    });
}
//# sourceMappingURL=index.js.map